﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;

namespace PinEncoding
{
    class pinEncoder
    {
        private string encodedPin;
        private byte[] encodedPinArr;

        public string EncodedPin { get => encodedPin; }
        public byte[] EncodedPinArr { get => encodedPinArr; }

        /// <summary>
        /// Encode PIN value and get a PIN block, explained in "PIN block encode.docx" file.
        /// </summary>
        /// <param name="pin"></param>
        public pinEncoder(string pin)
        {
            encodedPin = Encode(pin);
            encodedPinArr = Encoding.ASCII.GetBytes(EncodedPin);
        }

        private string Encode(string pin)
        {
            char[] encodedPinArray = AddPinPointer(pin);

            FillWithRandomValues(ref encodedPinArray);

            return new string(encodedPinArray);
        }
        /// <summary>
        /// Create random Hex and add pin at specific positions
        /// </summary>
        /// <param name="pin"></param>
        /// <returns></returns>
        private char[] AddPinPointer(string pin)
        {
            char[] encodedPinArray = new char[45];
            char[] pinArrHeader = new char[5];
            pinArrHeader[0] = Convert.ToChar(pin.Length.ToString());

            char[] pinChar = pin.ToCharArray();
            string random = RandomHex(pin.Length, true);

            for (int i = 0; i < pinChar.Length; i++)
            {
                var index = Convert.ToInt32(random[i].ToString(), 16);
                pinArrHeader[i + 1] = random[i];
                encodedPinArray[index] = pinChar[i];
            }
            var all = new char[pinArrHeader.Length + encodedPinArray.Length];
            pinArrHeader.CopyTo(all, 0);
            encodedPinArray.CopyTo(all, pinArrHeader.Length);
            return all;
        }

        /// <summary>
        /// fill the empty indixes with hex values.
        /// </summary>
        /// <param name="encodedPinArray"></param>
        private void FillWithRandomValues(ref char[] encodedPinArray)
        {
            string randomHex = RandomHex(50, false);

            for (int i = 0; i < randomHex.Length; i++)
            {
                if (encodedPinArray[i] == 0)
                {
                    encodedPinArray[i] = randomHex[i];
                }
            }
        }

        private string RandomHex(int charactersToGenarate, bool doubleCheck)
        {
            var csprng = new RNGCryptoServiceProvider();
            var bytes = new byte[charactersToGenarate / 2];

            csprng.GetNonZeroBytes(bytes);
            string randomHexString = string.Join("", bytes.Select(b => b.ToString("X2")));

            if (doubleCheck)
            {
                if (OnlyOnceCheck(randomHexString))
                {
                    return RandomHex(charactersToGenarate, doubleCheck);
                }
            }
            return randomHexString;
        }

        public bool OnlyOnceCheck(string input)
        {
            var set = new HashSet<char>();
            return input.Any(x => !set.Add(x));
        }

        private char[] RandomAsciiCharacter(int charactersToGenarate)
        {
            int asciiCharacterStart = 65; // from which ascii character code the generation should start
            int asciiCharacterEnd = 122; // to which ascii character code the generation should end

            Random random = new Random(DateTime.Now.Millisecond);
            char[] randomHex = new char[charactersToGenarate];
            // iterate, get random int between 'asciiCharacterStart' and 'asciiCharacterEnd', then convert it to (string), append to StringBuilder
            for (int i = 0; i < charactersToGenarate; i++)
            {
                randomHex[i] = ((char)(random.Next(asciiCharacterStart, asciiCharacterEnd + 1) % 255));
            }
            return randomHex;
        }

    }
}
