﻿using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Digests;
using Org.BouncyCastle.Crypto.Encodings;
using Org.BouncyCastle.Crypto.Engines;
using Org.BouncyCastle.Crypto.Modes;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.OpenSsl;
using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace PinEnc
{
    class Encryption
    {
        #region AES_GCM
        /// <summary>
        /// Encrypt a byte array with AES-GCM; Nonce is created randomly
        /// Ciphertext comes pre-concatenated with the authentication tag!!!
        /// </summary>
        /// <param name="messege">The array to encrypt</param>
        /// <param name="key">The key to encrypt it with</param>
        /// <param name="nonce">The nonce to encrypt it with (must be 12 bytes)</param>
        protected void AesGcmEncrypt(byte[] messege, EnrcyptedMessege enrcyptedMessege)
        {
            var cipher = new GcmBlockCipher(new AesEngine());
            cipher.Init(true, new AeadParameters(new KeyParameter(enrcyptedMessege.RandomAesKey), 128, enrcyptedMessege.RandomIv));
            var cipherBytes = new byte[cipher.GetOutputSize(messege.Length)];
            int len = cipher.ProcessBytes(messege, 0, messege.Length, cipherBytes, 0);
            cipher.DoFinal(cipherBytes, len);

            enrcyptedMessege.EncryptedMsg = cipherBytes;
            enrcyptedMessege.Tag = cipher.GetMac();

            //tag = nonce.Concat(cipherBytes).ToArray();
        }

        /// <summary>
        /// Generate a 96 bit nonce for Aes encryption
        /// </summary>
        /// <returns>nonce</returns>
        protected byte[] AesGenerateNonce()
        {
            byte[] nonce = new byte[12];
            new RNGCryptoServiceProvider().GetBytes(nonce);
            return nonce;
        }

        protected byte[] RandomByte(int charactersToGenarate)
        {
            var csprng = new RNGCryptoServiceProvider();
            var bytes = new byte[charactersToGenarate];

            csprng.GetNonZeroBytes(bytes);
            return bytes;
        }

        #endregion


        #region RSA_Oaep
        /// <summary>
        /// 
        /// </summary>
        /// <param name="clearText"></param>
        /// <param name="publicKey"></param>
        /// <returns>encodedBase64 enrypted string</returns>
        protected string RsaEncryptWithPublic(byte[] details, string publicKey)
        {
            // Read in public key from file
            var pemReader = new PemReader(new StringReader(publicKey));
            var rsaPub = (Org.BouncyCastle.Crypto.Parameters.RsaKeyParameters)pemReader.ReadObject();
            // create encrypter
            var encrypter = new OaepEncoding(new RsaEngine(), new Sha256Digest(), new Sha256Digest(), null);
            encrypter.Init(true, rsaPub);
            var cipher = encrypter.ProcessBlock(details, 0, details.Length);
            return Convert.ToBase64String(cipher);
        }
        #endregion

    }
}
