﻿using System;
using System.Linq;
using System.Text;

namespace PinEnc
{
    class PinEncryption : Encryption
    {
        private EnrcyptedMessege encryptedMessege;

        /// <summary>
        /// Generate AES KEY 256 bits, IV 96 bits and encrypt pin block with AES GCM.
        /// AES GCM has IV (nonce) with 12 bytes length.
        /// </summary>
        /// <param name="messege"></param>
        public PinEncryption(PinEncoder pEnc, PublicKey pKey,WeakPin weakPin)
        {
            encryptedMessege = new EnrcyptedMessege();

            if (weakPin.IsWeak(pEnc.Pin))
            {
                encryptedMessege.IsWeak = true;
                encryptedMessege.Info = weakPin.Info;
                return;
            }

            encryptedMessege.RandomAesKey =base.RandomByte(32);
            encryptedMessege.RandomIv = base.AesGenerateNonce();

            base.AesGcmEncrypt(pEnc.EncodedPinArr, encryptedMessege);

            // Copy encrypted message without tag
            Buffer.BlockCopy(encryptedMessege.EncryptedMsg, 0, encryptedMessege.EncryptedMsgWithoutIvAndAuthTag, 0, encryptedMessege.EncryptedMsgWithoutIvAndAuthTag.Length);

            encryptedMessege.EncodedMsgBase64 = Convert.ToBase64String(encryptedMessege.EncryptedMsgWithoutIvAndAuthTag);

            encryptedMessege.RsaEncodedDetailsbase64 = base.RsaEncryptWithPublic(BuildDetailsByte(), pKey.ValueDecoded);
        }

        public string GetMessege()
        {
            return encryptedMessege.EncodedMsgBase64;
        }
        public string GetDetails()
        {
            return encryptedMessege.RsaEncodedDetailsbase64;
        }
        public string GetInfo()
        {
           return encryptedMessege.Info;
        }
        public bool IsWeak()
        {
            return encryptedMessege.IsWeak;
        }

     
        /// <summary>
        ///Details have the following format:
        // first element is the key length as a character code
        // the next "key length" elements are the AES KEY
        // next element is the IV length as a character code
        // the next "IV length" elements are the AES IV
        // next element is the authentication tag length as a character code
        // the next "tag length" elements are the AES GCM authentication tag
        /// </summary>
        /// <returns></returns>
        private byte[] BuildDetailsByte()
        {
            byte[] keyPointer = new byte[1];
            keyPointer[0] = (byte)Convert.ToChar(encryptedMessege.RandomAesKey.Length);
            int kLengt = encryptedMessege.RandomAesKey.Length;

            byte[] ivPointer = new byte[1];
            ivPointer[0] = (byte)Convert.ToChar(encryptedMessege.RandomIv.Length);
            int ivLeng =encryptedMessege.RandomIv.Length;

            byte[] tagPointer = new byte[1];
            tagPointer[0] = (byte)Convert.ToChar(encryptedMessege.Tag.Length);
            int tagLeng =encryptedMessege.Tag.Length;

            byte[] details = new byte[kLengt + ivLeng + tagLeng + 3];

            details = keyPointer.Concat(encryptedMessege.RandomAesKey)
                .Concat(ivPointer)
                .Concat(encryptedMessege.RandomIv)
                .Concat(tagPointer)
                .Concat(encryptedMessege.Tag)
                .ToArray();

            return details;
        }
    }
}
